#!/usr/bin/python3
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Track list and helper functions."""
import attr

@attr.s(frozen=True)
class Track(object):
  name = attr.ib(type=str)
  start_finish = attr.ib(type=tuple)
  turns = attr.ib(type=tuple)


@attr.s(frozen=True)
class Turn(object):
  number = attr.ib(type=str)
  lat = attr.ib(type=float)
  lon = attr.ib(type=float)
  report_range = attr.ib(type=int, default=200)
